/*
 * Name:    offset_correct.c
 *
 * Purpose: Adjusts the offsets of an SMV image on a sector by sector basis.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "mx_util.h"
#include "mx_stdint.h"
#include "mx_image.h"

int
main( int argc, char *argv[] )
{
	MX_IMAGE_FRAME *frame;
	long num_sectors, num_sector_rows, num_sector_columns;
	long image_width, image_height, sector_width, sector_height;
	long n, i, j;
	char *input_filename, *output_filename;
	uint16_t *offset_array;
	uint16_t ***sector_array;
	void ***void_sector_array;
	mx_status_type mx_status;

	if ( argc < 6 ) {
		fprintf( stderr,
		"Usage: offset_correct num_sector_rows num_sector_columns "
		"input_filename output_filename "
		"sector_offset1 [ sector_offset2 ... ]\n" );
		exit(1);
	}

	num_sector_rows = atol( argv[1] );
	num_sector_columns = atol( argv[2] );

	input_filename = argv[3];
	output_filename = argv[4];

	frame = malloc( sizeof(MX_IMAGE_FRAME) );

	if ( frame == NULL ) {
		fprintf( stderr,
			"Could not allocate memory for an MX_IMAGE_FRAME.\n" );
		exit(1);
	}

	num_sectors = num_sector_rows * num_sector_columns;

	offset_array = malloc( num_sectors * sizeof(uint16_t) );

	if ( offset_array == NULL ) {
		fprintf( stderr,
			"Could not allocate memory for an offset array.\n" );
		exit(1);
	}

	for ( n = 0; n < num_sectors; n++ ) {
		offset_array[n] = atol( argv[5+n] );
	}

	/*---*/

	mx_status = mx_image_read_smv_file( &frame, input_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	image_width  = MXIF_ROW_FRAMESIZE(frame);
	image_height = MXIF_COLUMN_FRAMESIZE(frame);

	sector_width  = image_width  / num_sector_columns;
	sector_height = image_height / num_sector_rows;

	/*---*/

	mx_status = mx_image_alloc_sector_array( frame,
					num_sector_rows, num_sector_columns,
					&void_sector_array );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	sector_array = (uint16_t ***) void_sector_array;

	for ( n = 0; n < num_sectors; n++ ) {
		for ( i = 0; i < sector_height; i++ ) {
			for ( j = 0; j < sector_width; j++ ) {
				sector_array[n][i][j] += offset_array[n];
			}
		}
	}

	/*---*/

	mx_status = mx_image_write_smv_file( frame, output_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	exit(0);
}

